/*============================================================================
  File:     TailLogAfterMinLog.sql

  Summary:  This script shows a failed tail-log
			backup

  Date:     June 2010

  SQL Server Versions:
		10.0.2531.00 (SS2008 SP1)
		9.00.4035.00 (SS2005 SP3)
------------------------------------------------------------------------------
  Written by Paul S. Randal, SQLskills.com

  (c) 2011, SQLskills.com. All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you include this copyright and give due
  credit, but you must obtain prior permission before blogging this code.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

USE master;
GO

IF DATABASEPROPERTYEX ('DBMaint2008', 'Version') > 0
	DROP DATABASE DBMaint2008;
GO

-- Create the database
CREATE DATABASE DBMaint2008;
GO

USE DBMaint2008;
GO

ALTER DATABASE DBMaint2008
	SET RECOVERY FULL;
GO

-- Create a table
CREATE TABLE TestTable (
	c1 INT IDENTITY,
	c2 VARCHAR (100));
GO
CREATE CLUSTERED INDEX TestTable_CL
	ON TestTable (c1);
GO

INSERT INTO TestTable
	VALUES ('Initial data: transaction 1');
GO

-- And take a full backup
BACKUP DATABASE DBMaint2008 TO
	DISK = 'C:\SQLskills\DBMaint2008.bck'
WITH INIT;
GO

-- Now add some more data
SET NOCOUNT ON;
GO
INSERT INTO TestTable
	VALUES ('More data...');
GO 1000

BACKUP LOG DBMaint2008 TO
	DISK = 'C:\SQLskills\DBMaint2008_Log1.bck'
WITH INIT;
GO

-- Minimally-logged operation
ALTER DATABASE DBMaint2008
	SET RECOVERY BULK_LOGGED;
GO

ALTER INDEX TestTable_CL ON TestTable REBUILD;
GO

ALTER DATABASE DBMaint2008
	SET RECOVERY FULL;
GO

-- And some more user transactions
INSERT INTO TestTable
	VALUES ('Transaction 2');
GO
INSERT INTO TestTable
	VALUES ('Transaction 3');
GO

-- Simulate a crash
SHUTDOWN WITH NOWAIT;
GO

-- Trash the data file and restart SQL

USE DBMaint2008;
GO

-- The backup doesn't have the most recent
-- transactions - if we restore it we'll
-- lose them.

-- Take a log backup!
BACKUP LOG DBMaint2008 TO
	DISK = 'C:\SQLskills\DBMaint2008_tail.bck'
WITH INIT, NO_TRUNCATE;
GO

-- Hmm - marked as corrupt!

-- Now restore
RESTORE DATABASE DBMaint2008 FROM
	DISK = 'C:\SQLskills\DBMaint2008.bck'
WITH REPLACE, NORECOVERY;
GO

RESTORE LOG DBMaint2008 FROM
	DISK = 'C:\SQLskills\DBMaint2008_Log1.bck'
WITH REPLACE, NORECOVERY;
GO

RESTORE LOG DBMaint2008 FROM
	DISK = 'C:\SQLskills\DBMaint2008_tail.bck'
WITH REPLACE;
GO

-- Force it with CONTINUE_AFTER_ERROR
RESTORE LOG DBMaint2008 FROM
	DISK = 'C:\SQLskills\DBMaint2008_tail.bck'
WITH REPLACE, CONTINUE_AFTER_ERROR;
GO

-- Hmm - doesn't look good.

-- Is everything there?
SELECT * FROM DBMaint2008..TestTable;
GO

-- Woah!
DBCC CHECKDB (DBMaint2008) WITH NO_INFOMSGS;
GO

-- The tail-of-the-log backup is essentially corrupt because
-- it couldn't back up the data file pages

-- Re-restore without the tail-of-the-log
RESTORE DATABASE DBMaint2008 FROM
	DISK = 'C:\SQLskills\DBMaint2008.bck'
WITH REPLACE, NORECOVERY;
GO

RESTORE LOG DBMaint2008 FROM
	DISK = 'C:\SQLskills\DBMaint2008_Log1.bck'
WITH REPLACE;
GO

-- Is everything there?
SELECT * FROM DBMaint2008..TestTable;
GO

-- Nope - lost transaction 2 and 3
