/*============================================================================
  File:     SystemTableIndexCorruption.sql

  Summary:  This script demonstrates system table
			index corruption

  Date:     April 2011

  SQL Server Versions:
		10.0.2531.00 (SS2008 SP1)
------------------------------------------------------------------------------
  Written by Paul S. Randal, SQLskills.com

  (c) 2011, SQLskills.com. All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you include this copyright and give due
  credit, but you must obtain prior permission before blogging this code.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

/* Setup
USE master;
GO

-- Make sure the restore path exists...
RESTORE DATABASE DemoSystemIndex1 FROM
	DISK = 'D:\SQLskills\CorruptionDemoBackups\CorruptDemoSystemIndex1.bak'
WITH REPLACE, STATS = 10;
GO

RESTORE DATABASE DemoSystemIndex2 FROM
	DISK = 'D:\SQLskills\CorruptionDemoBackups\CorruptDemoSystemIndex2.bak'
WITH REPLACE, STATS = 10;
GO

RESTORE DATABASE DemoSystemIndex3 FROM
	DISK = 'D:\SQLskills\CorruptionDemoBackups\CorruptDemoSystemIndex3.bak'
WITH REPLACE, STATS = 10;
GO

RESTORE DATABASE DemoSystemIndex4 FROM
	DISK = 'D:\SQLskills\CorruptionDemoBackups\CorruptDemoSystemIndex4.bak'
WITH REPLACE, STATS = 10;
GO
*/

-- ** DemoSystemIndex1 **
-- What's wrong with this database?
DBCC CHECKDB (DemoSystemIndex1) WITH NO_INFOMSGS;
GO

-- System table corruption

-- What system tables are there?

USE DemoSystemIndex1;
GO

SELECT [name] FROM sys.objects;
GO

-- Try them in turn...
DBCC CHECKTABLE ('sys.sysrscols');
GO
DBCC CHECKTABLE ('sys.sysrowsets');
GO
DBCC CHECKTABLE ('sys.sysallocunits');
GO

-- Try to do a repair
ALTER DATABASE DemoSystemIndex1 SET SINGLE_USER;
GO

DBCC CHECKDB (DemoSystemIndex1, REPAIR_ALLOW_DATA_LOSS)
WITH NO_INFOMSGS;
GO

-- Same problem. Must use CHECKTABLE
DBCC CHECKTABLE ('sys.sysallocunits', REPAIR_ALLOW_DATA_LOSS)
WITH NO_INFOMSGS;
GO

-- Did it work?
DBCC CHECKTABLE ('sys.sysallocunits') WITH NO_INFOMSGS;
GO

-- ** DemoSystemIndex2 **
-- What's wrong with this database?
DBCC CHECKDB (DemoSystemIndex2) WITH NO_INFOMSGS;
GO

-- System table corruption

-- Try to do a repair
ALTER DATABASE DemoSystemIndex2 SET SINGLE_USER;
GO

DBCC CHECKDB (DemoSystemIndex2, REPAIR_ALLOW_DATA_LOSS)
WITH NO_INFOMSGS;
GO

-- CHECKDB will not rebuild system table clustered
-- indexes from b-tree errors

-- ** DemoSystemIndex3 **
-- What's wrong with this database?
DBCC CHECKDB (DemoSystemIndex3) WITH NO_INFOMSGS;
GO

-- System table corruption

-- Try to do a repair
ALTER DATABASE DemoSystemIndex3 SET SINGLE_USER;
GO

DBCC CHECKDB (DemoSystemIndex3, REPAIR_ALLOW_DATA_LOSS)
WITH NO_INFOMSGS;
GO

-- Oops. Found an engine bug.


-- Restart and try again.

-- CHECKDB will not rebuild system table clustered
-- indexes from b-tree errors

-- ** DemoSystemIndex4 **
-- What's wrong with this database?
DBCC CHECKDB (DemoSystemIndex4) WITH NO_INFOMSGS;
GO

-- System table corruption

-- Try to do a repair
ALTER DATABASE DemoSystemIndex4 SET SINGLE_USER;
GO

DBCC CHECKDB (DemoSystemIndex4, REPAIR_ALLOW_DATA_LOSS)
WITH NO_INFOMSGS;
GO

-- Did it fix it?
DBCC CHECKDB (DemoSystemIndex4) WITH NO_INFOMSGS;
GO

-- Oops. Found another engine bug.
-- This one means the clustered index metadata is broken
-- and was broken by CHECKDB trying to repair it.
