/*============================================================================
  File:     FixCorruptSystemTables.sql

  Summary:  This script shows fixing corrupt system tables
  
  Reference: http://www.sqlskills.com/BLOGS/PAUL/post/Disaster-recovery-101-fixing-metadata-corruption-without-a-backup.aspx

  Date:     March 2011

  SQL Server Versions:
		10.0.2531.00 (SS2008 SP1)
------------------------------------------------------------------------------
  Written by Paul S. Randal, SQLskills.com

  (c) 2011, SQLskills.com. All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you include this copyright and give due
  credit, but you must obtain prior permission before blogging this code.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

USE master;
GO

RESTORE DATABASE DemoCorruptSystemTables
FROM DISK ='D:\SQLskills\CorruptionDemoBackups\CorruptDemoCorruptSystemTables.bak'
WITH REPLACE;
GO

-- What's wrong with this database?
DBCC CHECKDB (DemoCorruptSystemTables) WITH NO_INFOMSGS;
GO

-- So we try DBCC CHECKCATALOG
DBCC CHECKCATALOG (DemoCorruptSystemTables) WITH NO_INFOMSGS;
GO 

-- Explain about warning at top of output

DBCC TRACEON (3604);
DBCC DBINFO (DemoCorruptSystemTables);
GO

/*
 Errors reference:
	- sys.indexes is really sys.sysidxstats
	- sys.objects is really sys.sysschobjs
	- sys.columns is really sys.syscolpars 

 Errors mean:
	1.There's an extra row in sys.syscolpars 
	2.There's an extra row in sys.sysidxstats 
	3.There's an extra row in sys.sysidxstats 
	4.There's an extra row in sys.sysschobjs
*/

-- To fix the errors:
USE DemoCorruptSystemTables;
GO 

DELETE FROM sys.syscolpars WHERE ID=2105058535;
GO
DELETE FROM sys.sysidxstats WHERE ID=2105058535;
GO
DELETE FROM sys.sysschobjs WHERE ID=5575058;
GO

-- Need to boot single-user mode and use the DAC


